;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x992660bcadf32def1a64d9feb5d4dabd . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe9fbbf4150041ded1992d41ac5fdf16e . "./code.cg2js/cg-gesture-emulation.fasl")
(#x841fce429a5a18fbdf5e0ae02a1753f1 . "./code.cg2js/cg-grid-widget.fasl")
(#x6563ad51d81744feca19fe1d47c79f26 . "./code.cg2js/cg-utility-dialog.fasl")
(#x20e87a8aae8b20fa3d50bc72232e534d . "./code.cg2js/cg-wrap-string.fasl")
(#x793728cab70fc7561aadd531789bd3b4 . "./code.cg2js/cg-editable-text.fasl")
(#xdea4b69184348f59ab58f73577193ef3 . "./code.cg2js/cg-base.fasl")
(#x7f41675d22d85d169c5c7eaf2f2e77d0 . "./code.cg2js/cg-lisp-text.fasl")
(#xd16b7e03a09c6e60dc9b9a4b230f36f3 . "./code.cg2js/cg-intersect.fasl")
(#xe021fe67577b44f1e977677ca089992d . "./code.cg2js/cg-choose-printer.fasl")
(#xf960812fec64def747aa37c097fec3dd . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x67fc39201862e80dd805bb819345c458 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x34141696511016ecb1bd8d20b0b68365 . "./code.cg2js/cg-font-dialog.fasl")
(#x1f1e5bb0ad137d6864ea564c5dc60517 . "./code.cg2js/cg-comtab.fasl")
(#x2ba6ea1a7d9bd8755c733913e34750ac . "./code.cg2js/cg-tooltip.fasl")
(#x7b84f8d5aaf3671793a890fe4cd3af9e . "./code.cg2js/cg-chart-or-plot.fasl")
(#x64785473aa8511b6b8226b08f7b8cef1 . "./code.cg2js/cg-get-pixmap.fasl")
(#x5e8c545a9201ea03ad28dabbe8a5dfeb . "./code.cg2js/cg-status-bar.fasl")
(#x5d885cfa63bf76fffb95286e00d63fb9 . "./code.cg2js/cg-tab-control.fasl")
(#x6ad85a1e426561532b80c875b2de126c . "./code.cg2js/cg-icon-pixmap.fasl")
(#x55b642e3399954c56f531927dba951b8 . "./code.cg2js/cg-list-view.fasl")
(#x85b9d0425485cf2f054efc6a13296ebf . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xc496be3bc1119897f99aa65aaf84734e . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xba6f00d48c812b2c24446440867c6785 . "./code.cg2js/cg-project-window.fasl")
(#x3aa4c572501a5a3157bf04bf1476347c . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xdb3627c78bcfacc0fe538d337d72feaa . "./code.cg2js/cg-drag-and-drop.fasl")
(#x084be8f0985ada9f99d204cc93ae897f . "./code.cg2js/cg-scaling-stream.fasl")
(#xf2c4611dacc503861a536cbd337fe0e8 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x47e3051abef1c2d463385d49eee1bf26 . "./code.cg2js/cg-menu.fasl")
(#xdf7196d4e0e666c3f5b2a2a8dfc69268 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xddd1f99c66cea5a8651bcf3e98e7c197 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xb7f271e6291096215c8f8fc3eef217ea . "./code.cg2js/cg-split-bar.fasl")
(#xba9cc4f78f1cf0320350595f08705620 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x6b563b0e32821e44428e8a1a0ab173a5 . "./code.cg2js/cg-timer.fasl")
(#xddda37979013686d5458408780120c8e . "./code.cg2js/cg-text-or-combo.fasl")
(#xa3528591815761c6093cf73a34797384 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xb9d15dc6a9b790f71eab6fc4476b7812 . "./code.cg2js/cg-string-dialog.fasl")
(#x6751310f7b8e21e3ef591d06dd028258 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x6cf831c5f78387d152e5b713ca721a75 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xa8906e9fdfc5e5fb2458fff658695278 . "./code.cg2js/cg-outline.fasl")
(#x9c674229e401483dc7961170fe84c0a7 . "./code.cg2js/cg-lettered-menu.fasl")
(#xa9bd538ed5c58fe69b4ca1a09205701c . "./code.cg2js/cg-graphics-context.fasl")
(#xc42599dbcbda75b013c63fe34a14d467 . "./code.cg2js/cg-item-list.fasl")
(#x24b6e3541e80cfc262dd933b7f116b14 . "./code.cg2js/cg-caret.fasl")
(#xd6fdb5606828e87e508e32fb8d24b06b . "./code.cg2js/cg-printing.fasl")
(#x69ee5bb53d38307f57648c097a1fdd9e . "./code.cg2js/cg-pixmap-printing.fasl")
(#x527c7c616fb6cd35423d22200db6bb02 . "./code.cg2js/cg-progress-indicator.fasl")
(#xaf8bbe97c4452bf2d55ab67c8d20ddb7 . "./code.cg2js/cg-dde-utils.fasl")
(#x0f39bd7b7c13a1bb083ec44ffff6f3c7 . "./code.cg2js/cg-plot-widget.fasl")
(#x56e6e16c12f16f7b6beeb7b7f8d7ac3d . "./code.cg2js/cg-os-window.fasl")
(#x4bba8b998416b7935d75f581be3b40e4 . "./code.cg2js/cg-calendar.fasl")
(#x873de41a56514463e0c37ec61202658b . "./code.cg2js/cg-chart-widget.fasl")
(#x51917b4aecb4cedf44bf8183ed0a8314 . "./code.cg2js/cg-file-dialog.fasl")
(#x98d06c6a0f9004abb67d1099c9b67cc6 . "./code.cg2js/cg-nodes.fasl")
(#x5af043920bcbc737b24a7abbefda82ec . "./code.cg2js/cg-lamp.fasl")
(#xe4333d733dddd15f5e91014a48b1c287 . "./code.cg2js/cg-html-dialog.fasl")
(#x1a3a0ed07bed87d47663d72a56c6e59e . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x0ffd39cb92acd433efe946c776aef8f5 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x0127e619d5e803e97754f7bfb694f013 . "./code.cg2js/cg-choice-list.fasl")
(#x71d816efdb6a4812daef6b7fdcc257e2 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xa26ed21dc8900a25b180656b1bbdf252 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xef57503865b63ca7fe9689ed89a0e03d . "./code.cg2js/cg-hotspot.fasl")
(#x56397cab463e26e1113c36f5df69f06d . "./code.cg2js/cg-property.fasl")
(#x6d647df768cd2de99d99dd66d0e35ece . "./code.cg2js/cg-dropping-outline.fasl")
(#xeecfd12a21b7dcf488442d95f489ad57 . "./code.cg2js/cg-template-string.fasl")
(#x38a1906bcd5217cae51eb8679eacba0f . "./code.cg2js/cg-palette.fasl")
(#x49ce7e0720035e22714ae773f8d0cc3a . "./code.cg2js/cg-selected-object.fasl")
(#x742a6b38a07ae445a2f754aaa475e949 . "./code.cg2js/cg-up-down-control.fasl")
(#x3fb48cb1584c534df176b3a52fe9e742 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x1abfe4ead43a39517c24a28026a0c097 . "./code.cg2js/cg-layout.fasl")
(#xed42fe2be4ed5006ea153d258eb4b554 . "./code.cg2js/cg-get-position.fasl")
(#xc76e7120e0151fae4f9e73e7276bb706 . "./code.cg2js/cg-lisp-widget.fasl")
(#x4357e03cf2568868352e5ba7013e810c . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xdd7bb1e0819ffb4ed90646237c0102cd . "./code.cg2js/cg-object-editor.fasl")
(#x0437e727c23edbce4323e0e1512f20a1 . "./code.cg2js/cg-class-grid.fasl")
(#x9e79a5357562ada4d03c01f61b0df387 . "./code.cg2js/cg-yes-no-string.fasl")
(#xb25219cfa966d93ac2a0c98337e24f77 . "./code.cg2js/cg-toggling-widget.fasl")
(#x0a1fbc758f7b4515cde919b7204d7105 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xf1a1b6a2e84b748c993355d92521d6a7 . "./code.cg2js/cg-button.fasl")
(#x2c99c47e31123770575fa2ca346683dc . "./code.cg2js/cg-object-editor-layout.fasl")
(#xc8232f1df7f3b8df633b81a8c39d2269 . "./code.cg2js/cg-scroll-bar.fasl")
(#x816f8f7ab9039609c95fc705c207afa5 . "./code.cg2js/cg-combo-box.fasl")
(#xa643292de9b3f90e2ff5ed608477f03c . "./code.cg2js/cg-web-browser.fasl")
(#x58b35c875fd7388fb2e424224bccc2fe . "./code.cg2js/cg-drawable.fasl")
(#x593c9146d688caea596cd1c841349ce6 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xeaea53471af5767525f8cf258ab23666 . "./code.cg2js/cg-message-dialog.fasl")
(#xe329619ccc986b97bfd1e9523fd24c49 . "./code.cg2js/cg-text-widget.fasl")
(#xae684c46138226abfa6f020977f659fd . "./code.cg2js/cg-static-text.fasl")
(#x81d2e15663f5d6b09fc3391e0c0b705d . "./code.cg2js/cg-class-support.fasl")
(#xeb75be7d92e2a444b85a9f53088d2495 . "./code.cg2js/cg.fasl")
(#x995210a304fa1e3294cf78eb6339bd91 . "./code.cg2js/cg-directory-dialog.fasl")
(#x696957ec68165be52d46396430cfb31c . "./code.cg2js/cg-clipboard.fasl")
(#x87e6109805ed7cf2c1d7d657508b8f48 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x9f116fc8a013bb7d0d4a2b64b5cd6cc8 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xb9fcece654eb546fe47966f5fe5445b6 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x6baa8710c2178e6a137f3552a61566af . "./code.cg2js/cg-check-box.fasl")
(#xdc28d0b94c09b2b9561efbdf104f96ee . "./code.cg2js/cg-color-dialog.fasl")
(#x87042976314d31f41d57d8d7a98825ee . "./code.cg2js/cg-text-edit-pane.fasl")
(#xbbb26773796bdb42b16f86c242b9441d . "./code.cg2js/cg-dialog-item.fasl")
(#x0360f76bc2117417b02f36b8a37bacd7 . "./code.cg2js/cg-edit-in-place.fasl")
(#x206b9e0c9c8292a842be4ee2c4658e85 . "./code.cg2js/cg-tray.fasl")
(#xf07953ba0fb05634bd08fce9d023ce9f . "./code.cg2js/cg-paren-matching.fasl")
(#x110cc8ff7e711aa4bccaa7e4e26b010c . "./code.cg2js/cg-player.fasl")
(#x400e3d09335d6efec12ad68d49effa4f . "./code.cg2js/cg-yes-no-list.fasl")
(#xfd4d1c0f42fdac49569f23f8a460d110 . "./code.cg2js/cg-common-control.fasl")
(#xa6c4031b4f4132ed34353ce6ec145ff2 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xfc3f6551e71fba3f6c3f2ad54330a89d . "./code.cg2js/cg-os-widget.fasl")
(#xc8954143af8b7bd210016434f2af99ba . "./code.cg2js/cg-sample-file-menu.fasl")
(#x77430f062d2151cb666ad277e4c7846f . "./code.cg2js/cg-trackbar.fasl")
(#x6f15ae0575b13b3014e364a1667fd9e6 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x5f8f0272bee95419eb3562381d3e07b7 . "./code.cg2js/cg-html-widget.fasl")
(#x0e098d463f1024c8eb7d8696aa573760 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xb0eab2e54bffd668dbbc740c598e035c . "./code.cg2js/cg-acache.fasl")
(#x890324b28a17b65eccbd840313a57aa9 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x41b818a14d2db89723dcb8d6ee714540 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xbcaf6572b458428b345c59a2272db263 . "./code.cg2js/cg-icon.fasl")
(#x3c52561ce35d1376c8197504cb8e2aec . "./code.cg2js/cg-pixmap.fasl")
(#x6b8b1a8b4b8ce214c936ee1625699e96 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x9a5b069cef5f73de192316aa1b868013 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xf2e0bfabdb4fa86ccc8bb310fff76a26 . "./code.cg2js/cg-toolbar.fasl")
(#x9eb3c4d7549f598d00ec4f99bc14ebac . "./code.cg2js/cg-radio-button.fasl")
(#x82573e5f43cd2ee0b399456de4dd5119 . "./code.cg2js/cg-curve.fasl")
(#xbe1bddb4d33c2d9148307581ec844154 . "./code.cg2js/cg-picture-widget.fasl")
(#x41df285ecc2e943fde620051db644100 . "./code.cg2js/cg-dev.fasl")
(#x8ab2b4023ee8cfc71f971f8fc5e69a9c . "./code.cg2js/cg-group-box.fasl")
(#xe82d709808c2ebec8d75bcc9961763a0 . "./code.cg2js/cg-fill-texture.fasl")
(#xd28e75c04747f2763327ae837bfeb844 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x57e277e79c5a1699e166cd95d027edb2 . "./code.idejs/ide-keyshort.fasl")
(#xae94164690f9e498bd11604f2e2c4621 . "./code.idejs/ide-form.fasl")
(#x49fdc0f8ea8de329e0bbd74514333d8d . "./code.idejs/ide-find-in-files.fasl")
(#x62882c04ee5d10196a434e26005d1e8c . "./code.idejs/ide-class-grapher.fasl")
(#x9ee5babdc636ed52828087b88a53c0c5 . "./code.idejs/ide-base.fasl")
(#x59f3cad260f0f47830a9095080a8791e . "./code.idejs/ide-class-browser.fasl")
(#x51d29f078cc2f98265bae5f9eff15f2f . "./code.idejs/ide-find-definitions.fasl")
(#xd4ace132e8eaed89cceb2ba0dac04082 . "./code.idejs/ide-package-list.fasl")
(#x0270529a2b8f9f9154aadc96b4911692 . "./code.idejs/ide-grapher.fasl")
(#x9fe7e147cee593f27f9f8174f615ccdd . "./code.idejs/ide-dev.fasl")
(#x875ec9cfb1590a6d287f5d05bab0a187 . "./code.idejs/ide.fasl")
(#x65a02d21f3dc01fcb8d610c7a95c6f98 . "./code.idejs/ide-trace.fasl")
(#x44c58256beadbe02077a7f0b0e7486f7 . "./code.idejs/ide-apropos.fasl")
(#xa341944e29f1981c57717a55030a9d90 . "./code.idejs/ide-inspector.fasl")
(#x236c7b7f3624c3e9cc87d4380eea8000 . "./code.idejs/ide-acache.fasl")
(#xda2491f4931c23f824db6cbb63e05f48 . "./code.idejs/ide-project.fasl")
(#x8aa289d15224c0f41d1544a8c37d3c1a . "./code.idejs/ide-window-list.fasl")
(#xd7bc83806f6634b3099c7dd3b1c64e55 . "./code.idejs/ide-intro.fasl")
(#x82416a720a47e1dd0ea449b8c5d63d18 . "./code.idejs/ide-clipboard.fasl")
(#x9c6141a29cef6a162da8b8a18a76a103 . "./code.idejs/ide-editor.fasl")
(#x208471d1017cd67583e36f24cfe638aa . "./code.idejs/ide-process-browser.fasl")
(#xae1f51cbf1efad314d4914655cbf66d7 . "./code.idejs/ide-patch-dialog.fasl")
(#x19b24c5c5457b9d260363bcaa6425835 . "./code.idejs/ide-navigator.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
